@@ -98,6 +98,11 @@ def parse_int(v: str) -> int:
         return 0
 
 
+def parse_heartbeat(v: str) -> int:
+    result = parse_int(v)
+    return result if 0 <= result < 65535 else 0
+
+
 def parse_connection_name(connection_name: Optional[str]) -> Dict[str, str]:
     if not connection_name or not isinstance(connection_name, str):
         return {}
@@ -238,7 +243,8 @@ class Connection(Base, AbstractConnection):
 
         self.last_channel = 1
 
-        self.heartbeat_timeout = parse_int(
+        self.timeout = parse_int(self.url.query.get("timeout", "60"))
+        self.heartbeat_timeout = parse_heartbeat(
             self.url.query.get("heartbeat", "60"),
         )
         self.last_channel_lock = asyncio.Lock()
@@ -364,7 +370,7 @@ class Connection(Base, AbstractConnection):
 
             frame_receiver = FrameReceiver(
                 reader,
-                (self.heartbeat_timeout + 1) * self.HEARTBEAT_GRACE_MULTIPLIER,
+                (self.timeout + 1) * self.HEARTBEAT_GRACE_MULTIPLIER,
             )
         except OSError as e:
             raise ConnectionError(*e.args) from e
@@ -403,9 +409,7 @@ class Connection(Base, AbstractConnection):
                 raise AMQPInternalError("Connection.Tune", frame)
 
             connection_tune: spec.Connection.Tune = frame
-
-            if self.heartbeat_timeout > 0:
-                connection_tune.heartbeat = self.heartbeat_timeout
+            connection_tune.heartbeat = self.heartbeat_timeout
 
             await self._rpc(
                 spec.Connection.TuneOk(
@@ -505,7 +509,7 @@ class Connection(Base, AbstractConnection):
         while not self.is_closed:
             try:
                 yield await asyncio.wait_for(
-                    self.write_queue.get(), timeout=self.heartbeat_timeout,
+                    self.write_queue.get(), timeout=self.timeout,
                 )
                 self.write_queue.task_done()
             except asyncio.TimeoutError:
@@ -527,12 +531,12 @@ class Connection(Base, AbstractConnection):
                     )
 
                     try:
-                        frame_bytes = pamqp.frame.marshal(
-                            frame, channel_frame.channel_number,
+                        writer.write(
+                            pamqp.frame.marshal(
+                                frame, channel_frame.channel_number,
+                            ),
                         )
-                        writer.write(frame_bytes)
-                        del frame_bytes
-                    except Exception as e:
+                    except BaseException as e:
                         log.exception(
                             "Failed to write frame to channel %d: %r",
                             channel_frame.channel_number,
@@ -543,7 +547,10 @@ class Connection(Base, AbstractConnection):
                     if isinstance(frame, spec.Connection.CloseOk):
                         return
 
-                    if channel_frame.drain_future is not None:
+                    if (
+                        channel_frame.drain_future is not None and
+                        not channel_frame.drain_future.done()
+                    ):
                         channel_frame.drain_future.set_result(
                             await writer.drain(),
                         )
