@@ -1,6 +1,10 @@
 import asyncio
 from functools import wraps
-from typing import AsyncContextManager, Awaitable, TypeVar
+from types import TracebackType
+from typing import (
+    Any, AsyncContextManager, Awaitable, Callable, Coroutine, Optional, Type,
+    TypeVar, Union,
+)
 
 from yarl import URL
 
@@ -10,57 +14,44 @@ from aiormq.abc import TimeoutType
 T = TypeVar("T")
 
 
-def censor_url(url: URL):
+def censor_url(url: URL) -> URL:
     if url.password is not None:
         return url.with_password("******")
     return url
 
 
-def shield(func):
+def shield(func: Callable[..., Awaitable[T]]) -> Callable[..., Awaitable[T]]:
     @wraps(func)
-    def wrap(*args, **kwargs):
-        return asyncio.shield(awaitable(func)(*args, **kwargs))
+    def wrap(*args: Any, **kwargs: Any) -> Awaitable[T]:
+        return asyncio.shield(func(*args, **kwargs))
 
     return wrap
 
 
-def awaitable(func):
+def awaitable(
+    func: Callable[..., Union[T, Awaitable[T]]],
+) -> Callable[..., Coroutine[Any, Any, T]]:
     # Avoid python 3.8+ warning
     if asyncio.iscoroutinefunction(func):
-        return func
+        return func     # type: ignore
 
     @wraps(func)
-    async def wrap(*args, **kwargs):
+    async def wrap(*args: Any, **kwargs: Any) -> T:
         result = func(*args, **kwargs)
 
         if hasattr(result, "__await__"):
-            return await result
+            return await result     # type: ignore
         if asyncio.iscoroutine(result) or asyncio.isfuture(result):
-            return await result
+            return await result     # type: ignore
 
-        return result
+        return result               # type: ignore
 
     return wrap
 
 
-def _inspect_await_method():
-    async def _test():
-        pass
-
-    coro = _test()
-    method_await = getattr(coro, "__await__", None)
-    method_iter = getattr(coro, "__iter__", None)
-
-    for _ in (method_await or method_iter)():
-        pass
-
-    return bool(method_await)
-
-
-HAS_AWAIT_METHOD = _inspect_await_method()
-
-
 class Countdown:
+    __slots__ = "loop", "deadline"
+
     def __init__(self, timeout: TimeoutType = None):
         self.loop: asyncio.AbstractEventLoop = asyncio.get_event_loop()
         self.deadline: TimeoutType = None
@@ -94,12 +85,15 @@ class CountdownContext(AsyncContextManager):
         self.countdown = countdown
         self.ctx = ctx
 
-    def __aenter__(self):
+    def __aenter__(self) -> Awaitable[T]:
         if self.countdown.deadline is None:
             return self.ctx.__aenter__()
         return self.countdown(self.ctx.__aenter__())
 
-    def __aexit__(self, exc_type, exc_val, exc_tb):
+    def __aexit__(
+        self, exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException], exc_tb: Optional[TracebackType],
+    ) -> Awaitable[Any]:
         if self.countdown.deadline is None:
             return self.ctx.__aexit__(exc_type, exc_val, exc_tb)
 
